import socket

# Constants
HOST_PORT = 8410
MBOX_PORT = 7408
LISTEN_PORT = 7407
DEST_IP = '192.168.1.69'

CONFIRM_CODE = bytes([0x55, 0xAA])
PASSS_CODE = bytes([0x00, 0x00])
FUNCTION_CODE = bytes([0x14, 0x01])
OBJECT_CHANNEL = bytes([0x00, 0x00])
WHO_ACCEPT = bytes([0xFF, 0xFF])
WHO_REPLY = bytes([0xFF, 0xFF])
LINE = bytes([0x00, 0x00, 0x00, 0x01])
DELTA_TIME = bytes([0x00, 0x00, 0x00, 0x00])
BASE_DOUT = bytes([0x00, 0x00])
DAC = bytes([0x00, 0x00, 0x00, 0x00])
HEADER_BYTES = 20
AXIS_DATA_LENGTH = 4
FOOTER_BYTES = 16

LINE_COUNTER = 1

GOTO_MAX = bytes([
    0x55, 0xaa, 0x00, 0x00, 0x14, 0x01,
    0x00, 0x00, 0xff, 0xff, 0xff, 0xff, 0x00, 0x00, 0x00, 0x01, 0x00, 0x00, 0x00, 0x00, 0x00, 0x01,
    0x86, 0xa0, 0x00, 0x01, 0x86, 0xa0, 0x00, 0x01, 0x86, 0xa0, 0x00, 0x01, 0x86, 0xa0, 0x00, 0x01,
    0x86, 0xa0, 0x00, 0x01, 0x86, 0xa0, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00
])

GOTO_ZERO = bytes([
    0x55, 0xaa, 0x00, 0x00, 0x14, 0x01,
    0x00, 0x00, 0xff, 0xff, 0xff, 0xff, 0x00, 0x00, 0x00, 0x01, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
    0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
    0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00
])

def create_message_bytes(axis_data_bytes):
    global LINE_COUNTER
    global LINE

    header = CONFIRM_CODE + PASSS_CODE + FUNCTION_CODE + OBJECT_CHANNEL + WHO_ACCEPT + WHO_REPLY + LINE + DELTA_TIME
    footer = BASE_DOUT + DAC
    axis_data_byte_string = b''.join(axis_data_bytes)

    # Combine header, axis data bytes, and footer to create the message
    message = header + axis_data_byte_string + footer

    # Increment the LINE counter for the next message
    LINE_COUNTER += 1
    # LINE = LINE_COUNTER.to_bytes(4, 'big')

    return message

def goto_max():
    print(GOTO_MAX);
    send_message(GOTO_MAX)

def goto_zero():
    print(GOTO_ZERO);
    send_message(GOTO_ZERO)

def send_message(message):
    with socket.socket(socket.AF_INET, socket.SOCK_DGRAM) as sock:
        sock.bind(('', HOST_PORT))
        sock.sendto(message, (DEST_IP, MBOX_PORT))

def udp_listener():
    sock = socket.socket(socket.AF_INET, socket.SOCK_DGRAM)
    sock.bind(('', LISTEN_PORT))

    while True:
        print('\nwaiting to receive message')
        data, address = sock.recvfrom(4096)

        print(f'RAW DATA: {data}\n')
        print(f'received {len(data)} bytes from {address}')

        if data:
            # Calculate the number of axes (assuming each axis is represented by 2 bytes)
            num_axes = len(data)

            # Initialize an empty list to store the axis values
            axis_values = []
            axis_data_bytes = []

            # Loop over the data and extract each axis value
            for i in range(num_axes):
                # Extract the data for the current axis
                axis_data = data[i:(i+1)]
                axis_value = (int.from_bytes(axis_data, 'little')/127)*100000

                # Convert the data to an integer and add it to the list
                axis_values.append(axis_value)

                axis_bytes = int(axis_value).to_bytes(4, 'big')
                # axis_bytes = axis_bytes[-1:] + axis_bytes[1:3] + axis_bytes[:1]
                axis_data_bytes.append(axis_bytes)


            # Print the values of all axes
            for i, value in enumerate(axis_values):
                print(f"Axis {i+1} value: {value}")
                print(f"Axis {i+1} bytes: {axis_data_bytes[i]}")

            # Create the message bytes with the axis data bytes
            message_bytes = create_message_bytes(axis_data_bytes)

            print(f"Message bytes: {message_bytes}")

            # Send the message
            send_message(message_bytes)

# Start UDP listener
udp_listener()
# goto_max()
# goto_zero()